@@ -28,8 +28,6 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus-glib.h>
 
-#define TRACKER_TYPE_STR_STRV_MAP (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_STRV))
-
 G_BEGIN_DECLS
 
 #if !defined (__LIBTRACKER_COMMON_INSIDE__) && !defined (TRACKER_COMPILATION)
@@ -39,17 +37,30 @@ G_BEGIN_DECLS
 #define TRACKER_DBUS_ERROR_DOMAIN "TrackerDBus"
 #define TRACKER_DBUS_ERROR        tracker_dbus_error_quark()
 
-#define TRACKER_TYPE_EVENT_ARRAY	  \
+
+#define TRACKER_TYPE_INT_ARRAY_MAP	\
+	dbus_g_type_get_map ("GHashTable", G_TYPE_INT, DBUS_TYPE_G_INT_ARRAY)
+
+#define TRACKER_TYPE_FOUR_INT_ARRAY	\
+	dbus_g_type_get_collection ("GPtrArray", \
+	                            dbus_g_type_get_struct("GValueArray", \
+	                                                    G_TYPE_INT, \
+	                                                    G_TYPE_INT, \
+	                                                    G_TYPE_INT, \
+	                                                    G_TYPE_INT, \
+	                                                    G_TYPE_INVALID))
+
+#define TRACKER_TYPE_EVENT_ARRAY	\
 	dbus_g_type_get_collection ("GPtrArray", \
 	                            dbus_g_type_get_struct ("GValueArray", \
 	                                                    G_TYPE_STRING, \
 	                                                    G_TYPE_STRING, \
 	                                                    G_TYPE_INT, \
 	                                                    G_TYPE_INVALID))
-#define TRACKER_TYPE_G_STRV_ARRAY	  \
+#define TRACKER_TYPE_G_STRV_ARRAY	\
 	dbus_g_type_get_collection ("GPtrArray", G_TYPE_STRV)
 
-#define tracker_dbus_async_return_if_fail(expr,context)	  \
+#define tracker_dbus_async_return_if_fail(expr,context)	\
 	G_STMT_START { \
 		if G_LIKELY(expr) { } else { \
 			GError *assert_error = NULL; \
@@ -67,7 +78,7 @@ G_BEGIN_DECLS
 		}; \
 	} G_STMT_END
 
-#define tracker_dbus_return_val_if_fail(expr,val,error)	  \
+#define tracker_dbus_return_val_if_fail(expr,val,error)	\
 	G_STMT_START { \
 		if G_LIKELY(expr) { } else { \
 			g_set_error (error, \
