@@ -33,14 +33,13 @@
 #include <libtracker-fts/tracker-fts.h>
 #endif
 
-#include <libtracker-db/tracker-db-manager.h>
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-journal.h>
-
 #include "tracker-class.h"
 #include "tracker-data-manager.h"
 #include "tracker-data-update.h"
 #include "tracker-data-query.h"
+#include "tracker-db-manager.h"
+#include "tracker-db-dbus.h"
+#include "tracker-db-journal.h"
 #include "tracker-ontologies.h"
 #include "tracker-property.h"
 #include "tracker-sparql-query.h"
@@ -334,10 +333,6 @@ tracker_data_remove_delete_statement_callback (TrackerStatementCallback callback
 	}
 }
 
-GQuark tracker_data_error_quark (void) {
-	return g_quark_from_static_string ("tracker_data_error-quark");
-}
-
 static gint
 tracker_data_update_get_new_service_id (void)
 {
@@ -361,7 +356,7 @@ tracker_data_update_get_new_service_id (void)
 	}
 
 	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, &error)) {
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			max_service_id = MAX (tracker_db_cursor_get_int (cursor, 0), max_service_id);
 		}
 
@@ -406,7 +401,7 @@ tracker_data_update_get_next_modseq (void)
 	}
 
 	if (cursor) {
-		if (tracker_db_cursor_iter_next (cursor, &error)) {
+		if (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 			max_modseq = MAX (tracker_db_cursor_get_int (cursor, 0), max_modseq);
 		}
 
@@ -870,6 +865,10 @@ tracker_data_resource_buffer_flush (GError **error)
 							 tracker_data_query_resource_id (tracker_property_get_uri (prop)),
 							 fts->str, !tracker_property_get_fulltext_no_limit (prop));
 				g_string_free (fts, TRUE);
+
+				/* Set that we ever updated FTS, so that tracker_fts_update_commit()
+				 * gets called */
+				update_buffer.fts_ever_updated = TRUE;
 			}
 		}
 	}
@@ -1036,7 +1035,7 @@ cache_create_service_decomposed (TrackerClass *cl,
 {
 	TrackerClass       **super_classes;
 	GValue              gvalue = { 0 };
-	gint                i;
+	gint                i, final_graph_id, class_id;
 
 	/* also create instance of all super classes */
 	super_classes = tracker_class_get_super_classes (cl);
@@ -1058,24 +1057,31 @@ cache_create_service_decomposed (TrackerClass *cl,
 
 	cache_insert_row (cl);
 
-	g_value_set_int64 (&gvalue, ensure_resource_id (tracker_class_get_uri (cl), NULL));
+	final_graph_id = (graph != NULL ? ensure_resource_id (graph, NULL) : graph_id);
+
+	/* This is the original, no idea why tracker_class_get_id wasn't used here:
+	 * class_id = ensure_resource_id (tracker_class_get_uri (cl), NULL); */
+
+	class_id = tracker_class_get_id (cl);
+
+	g_value_set_int64 (&gvalue, class_id);
 	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue,
-	                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
+	                    final_graph_id,
 	                    TRUE, FALSE, FALSE);
 
 	add_class_count (cl, 1);
+
 	if (!in_journal_replay && insert_callbacks) {
 		guint n;
-		const gchar *class_uri;
-
-		class_uri = tracker_class_get_uri (cl);
 
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
 
 			delegate = g_ptr_array_index (insert_callbacks, n);
-			delegate->callback (graph, resource_buffer->subject,
-			                    RDF_PREFIX "type", class_uri,
+			delegate->callback (final_graph_id, graph, resource_buffer->id, resource_buffer->subject,
+			                    tracker_property_get_id (tracker_ontologies_get_rdf_type ()),
+			                    class_id,
+			                    tracker_class_get_uri (cl),
 			                    resource_buffer->types,
 			                    delegate->user_data);
 		}
@@ -1247,7 +1253,7 @@ get_old_property_values (TrackerProperty  *property,
 	old_values = g_hash_table_lookup (resource_buffer->predicates, property);
 	if (old_values == NULL) {
 		if (!check_property_domain (property)) {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_CONSTRAINT,
 			             "Subject `%s' is not in domain `%s' of property `%s'",
 			             resource_buffer->subject,
 			             tracker_class_get_name (tracker_property_get_domain (property)),
@@ -1348,6 +1354,18 @@ string_to_gvalue (const gchar         *value,
 	}
 }
 
+static gboolean
+resource_in_domain_index_class (TrackerClass *domain_index_class)
+{
+	guint i;
+	for (i = 0; i < resource_buffer->types->len; i++) {
+		if (g_ptr_array_index (resource_buffer->types, i) == domain_index_class) {
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static gboolean
 cache_set_metadata_decomposed (TrackerProperty  *property,
                                const gchar      *value,
@@ -1369,7 +1387,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 	super_properties = tracker_property_get_super_properties (property);
 	while (*super_properties) {
 		change |= cache_set_metadata_decomposed (*super_properties, value, value_id,
-		                               graph, graph_id, &new_error);
+		                                         graph, graph_id, &new_error);
 		if (new_error) {
 			g_propagate_error (error, new_error);
 			return FALSE;
@@ -1421,7 +1439,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 			new_value_str = g_value_get_string (&new_value);
 		}
 
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_CONSTRAINT,
+		g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_CONSTRAINT,
 		             "Unable to insert multiple values for subject `%s' and single valued property `%s' "
 		             "(old_value: '%s', new value: '%s')",
 		             resource_buffer->subject,
@@ -1434,12 +1452,34 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		g_value_unset (&gvalue);
 
 	} else {
+
 		cache_insert_value (table_name, field_name, &gvalue,
 		                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
 		                    multiple_values,
 		                    tracker_property_get_fulltext_indexed (property),
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 
+		if (!multiple_values) {
+			TrackerClass **domain_index_classes;
+
+			domain_index_classes = tracker_property_get_domain_indexes (property);
+			while (*domain_index_classes) {
+				if (resource_in_domain_index_class (*domain_index_classes)) {
+					GValue gvalue_copy = { 0 };
+
+					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+					g_value_copy (&gvalue, &gvalue_copy);
+
+					cache_insert_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy,
+					                    graph != NULL ? ensure_resource_id (graph, NULL) : graph_id,
+					                    multiple_values,
+					                    tracker_property_get_fulltext_indexed (property),
+					                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+				}
+				domain_index_classes++;
+			}
+		}
+
 		change = TRUE;
 	}
 
@@ -1491,6 +1531,24 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		                    tracker_property_get_fulltext_indexed (property),
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
 
+		if (!multiple_values) {
+			TrackerClass **domain_index_classes;
+
+			domain_index_classes = tracker_property_get_domain_indexes (property);
+
+			while (*domain_index_classes) {
+				if (resource_in_domain_index_class (*domain_index_classes)) {
+					GValue gvalue_copy = { 0 };
+					g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+					g_value_copy (&gvalue, &gvalue_copy);
+					cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+					                    tracker_property_get_fulltext_indexed (property),
+					                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+				}
+				domain_index_classes++;
+			}
+		}
+
 		change = TRUE;
 	}
 
@@ -1599,6 +1657,25 @@ cache_delete_resource_type (TrackerClass *class,
 			cache_delete_value (table_name, field_name, &gvalue, multiple_values,
 			                    tracker_property_get_fulltext_indexed (prop),
 			                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+
+			if (!multiple_values) {
+				TrackerClass **domain_index_classes;
+
+				domain_index_classes = tracker_property_get_domain_indexes (prop);
+				while (*domain_index_classes) {
+					if (resource_in_domain_index_class (*domain_index_classes)) {
+						GValue gvalue_copy = { 0 };
+						g_value_init (&gvalue_copy, G_VALUE_TYPE (&gvalue));
+						g_value_copy (&gvalue, &gvalue_copy);
+						cache_delete_value (tracker_class_get_name (*domain_index_classes), field_name, &gvalue_copy, multiple_values,
+						                    tracker_property_get_fulltext_indexed (prop),
+						                    tracker_property_get_data_type (prop) == TRACKER_PROPERTY_TYPE_DATETIME);
+					}
+					domain_index_classes++;
+				}
+			}
+
 		}
 	}
 
@@ -1606,13 +1683,20 @@ cache_delete_resource_type (TrackerClass *class,
 
 	if (!in_journal_replay && delete_callbacks) {
 		guint n;
+		gint final_graph_id;
+
+		final_graph_id = (graph != NULL ? ensure_resource_id (graph, NULL) : graph_id);
+
 		for (n = 0; n < delete_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
 
 			delegate = g_ptr_array_index (delete_callbacks, n);
-			delegate->callback (graph, resource_buffer->subject,
-			                    RDF_PREFIX "type", tracker_class_get_uri (class),
-			                    resource_buffer->types, delegate->user_data);
+			delegate->callback (final_graph_id, graph, resource_buffer->id, resource_buffer->subject,
+			                    tracker_property_get_id (tracker_ontologies_get_rdf_type ()),
+			                    tracker_class_get_id (class),
+			                    tracker_class_get_uri (class),
+			                    resource_buffer->types,
+			                    delegate->user_data);
 		}
 	}
 
@@ -1697,8 +1781,8 @@ tracker_data_delete_statement (const gchar  *graph,
 {
 	TrackerClass       *class;
 	TrackerProperty    *field;
-	gint                subject_id;
-	gboolean change = FALSE;
+	gint                subject_id = 0;
+	gboolean            change = FALSE;
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
@@ -1719,52 +1803,71 @@ tracker_data_delete_statement (const gchar  *graph,
 		if (class != NULL) {
 			if (!in_journal_replay) {
 				tracker_db_journal_append_delete_statement_id (
-					(graph != NULL ? query_resource_id (graph) : 0),
-					resource_buffer->id,
-					tracker_data_query_resource_id (predicate),
-					query_resource_id (object));
+				       (graph != NULL ? query_resource_id (graph) : 0),
+				       resource_buffer->id,
+				       tracker_data_query_resource_id (predicate),
+				       tracker_class_get_id (class));
 			}
-
 			cache_delete_resource_type (class, graph, 0);
 		} else {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_CLASS,
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_CLASS,
 			             "Class '%s' not found in the ontology", object);
 		}
 	} else {
+		gint pred_id = 0, graph_id = 0, object_id = 0;
+		gboolean tried = FALSE;
+
 		field = tracker_ontologies_get_property_by_uri (predicate);
 		if (field != NULL) {
-			gint id;
-
 			change = delete_metadata_decomposed (field, object, 0, error);
-
-			id = tracker_property_get_id (field);
 			if (!in_journal_replay && change) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
-					tracker_db_journal_append_delete_statement_id (
-						(graph != NULL ? query_resource_id (graph) : 0),
-						resource_buffer->id,
-						(id != 0) ? id : tracker_data_query_resource_id (predicate),
-						query_resource_id (object));
+
+					pred_id = tracker_property_get_id (field);
+					graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+					object_id = query_resource_id (object);
+					tried = TRUE;
+
+					tracker_db_journal_append_delete_statement_id (graph_id,
+					                                               resource_buffer->id,
+					                                               pred_id,
+					                                               object_id);
 				} else {
-					tracker_db_journal_append_delete_statement (
-						(graph != NULL ? query_resource_id (graph) : 0),
-						resource_buffer->id,
-						(id != 0) ? id : tracker_data_query_resource_id (predicate),
-						object);
+					pred_id = tracker_property_get_id (field);
+					graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+					object_id = 0;
+					tried = TRUE;
+
+					tracker_db_journal_append_delete_statement (graph_id,
+					                                            resource_buffer->id,
+					                                            pred_id,
+					                                            object);
 				}
 			}
 		} else {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
+			/* I wonder why in case of error the delete_callbacks are still executed */
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY,
 			             "Property '%s' not found in the ontology", predicate);
 		}
 
+		if (!tried) {
+			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+			if (field == NULL) {
+				pred_id = tracker_data_query_resource_id (predicate);
+			} else {
+				pred_id = tracker_property_get_id (field);
+			}
+		}
+
 		if (delete_callbacks && change) {
 			guint n;
 			for (n = 0; n < delete_callbacks->len; n++) {
 				TrackerStatementDelegate *delegate;
 
 				delegate = g_ptr_array_index (delete_callbacks, n);
-				delegate->callback (graph, subject, predicate, object,
+				delegate->callback (graph_id, graph, subject_id, subject,
+				                    pred_id, object_id,
+				                    object,
 				                    resource_buffer->types,
 				                    delegate->user_data);
 			}
@@ -1844,7 +1947,7 @@ tracker_data_insert_statement (const gchar            *graph,
 			tracker_data_insert_statement_with_string (graph, subject, predicate, object, error);
 		}
 	} else {
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
+		g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
 	}
 }
@@ -1859,7 +1962,8 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 	GError          *actual_error = NULL;
 	TrackerClass    *class;
 	TrackerProperty *property;
-	gint             prop_id = 0;
+	gint             prop_id = 0, graph_id = 0;
+	gint             final_prop_id = 0, object_id = 0;
 	gboolean change = FALSE;
 
 	g_return_if_fail (subject != NULL);
@@ -1869,12 +1973,12 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 
 	property = tracker_ontologies_get_property_by_uri (predicate);
 	if (property == NULL) {
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
+		g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
 		return;
 	} else {
 		if (tracker_property_get_data_type (property) != TRACKER_PROPERTY_TYPE_RESOURCE) {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_INVALID_TYPE,
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_TYPE,
 			             "Property '%s' does not accept URIs", predicate);
 			return;
 		}
@@ -1933,11 +2037,17 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		if (class != NULL) {
 			cache_create_service_decomposed (class, graph, 0);
 		} else {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_CLASS,
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_CLASS,
 			             "Class '%s' not found in the ontology", object);
 			return;
 		}
 
+		if (!in_journal_replay) {
+			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+			final_prop_id = (prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate);
+			object_id = query_resource_id (object);
+		}
+
 		change = TRUE;
 	} else {
 		/* add value to metadata database */
@@ -1947,25 +2057,34 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 			return;
 		}
 
-		if (insert_callbacks && change) {
-			guint n;
-			for (n = 0; n < insert_callbacks->len; n++) {
-				TrackerStatementDelegate *delegate;
+		if (change) {
+			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+			final_prop_id = (prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate);
+			object_id = query_resource_id (object);
 
-				delegate = g_ptr_array_index (insert_callbacks, n);
-				delegate->callback (graph, subject, predicate, object,
-				                    resource_buffer->types,
-				                    delegate->user_data);
+			if (insert_callbacks) {
+				guint n;
+				for (n = 0; n < insert_callbacks->len; n++) {
+					TrackerStatementDelegate *delegate;
+
+					delegate = g_ptr_array_index (insert_callbacks, n);
+					delegate->callback (graph_id, graph, resource_buffer->id, subject,
+					                    final_prop_id, object_id,
+					                    object,
+					                    resource_buffer->types,
+					                    delegate->user_data);
+				}
 			}
 		}
+
 	}
 
 	if (!in_journal_replay && change) {
 		tracker_db_journal_append_insert_statement_id (
 			(graph != NULL ? query_resource_id (graph) : 0),
 			resource_buffer->id,
-			(prop_id != 0) ? prop_id : tracker_data_query_resource_id (predicate),
-			query_resource_id (object));
+			final_prop_id,
+			object_id);
 	}
 }
 
@@ -1978,8 +2097,8 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 {
 	GError          *actual_error = NULL;
 	TrackerProperty *property;
-	gint             id = 0;
-	gboolean change;
+	gboolean         change, tried = FALSE;
+	gint             graph_id = 0, pred_id = 0;
 
 
 	g_return_if_fail (subject != NULL);
@@ -1989,16 +2108,16 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 
 	property = tracker_ontologies_get_property_by_uri (predicate);
 	if (property == NULL) {
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
+		g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
 		return;
 	} else {
 		if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_RESOURCE) {
-			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_INVALID_TYPE,
+			g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_TYPE,
 			             "Property '%s' only accepts URIs", predicate);
 			return;
 		}
-		id = tracker_property_get_id (property);
+		pred_id = tracker_property_get_id (property);
 	}
 
 	if (!tracker_data_insert_statement_common (graph, subject, predicate, object, &actual_error)) {
@@ -2019,32 +2138,48 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 
 	if (insert_callbacks && change) {
 		guint n;
+
+		graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+		pred_id = (pred_id != 0) ? pred_id : tracker_data_query_resource_id (predicate);
+		tried = TRUE;
+
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
 
 			delegate = g_ptr_array_index (insert_callbacks, n);
-			delegate->callback (graph, subject, predicate, object,
+			delegate->callback (graph_id, graph, resource_buffer->id, subject,
+			                    pred_id, 0 /* Always a literal */,
+			                    object,
 			                    resource_buffer->types,
 			                    delegate->user_data);
 		}
 	}
 
 	if (!in_journal_replay && change) {
-		tracker_db_journal_append_insert_statement (
-			(graph != NULL ? query_resource_id (graph) : 0),
-			resource_buffer->id,
-			(id != 0) ? id : tracker_data_query_resource_id (predicate),
-			object);
+		if (!tried) {
+			graph_id = (graph != NULL ? query_resource_id (graph) : 0);
+			pred_id = (pred_id != 0) ? pred_id : tracker_data_query_resource_id (predicate);
+		}
+		tracker_db_journal_append_insert_statement (graph_id,
+		                                            resource_buffer->id,
+		                                            pred_id,
+		                                            object);
 	}
 }
 
 void
-tracker_data_begin_db_transaction (void)
+tracker_data_begin_transaction (GError **error)
 {
 	TrackerDBInterface *iface;
 
 	g_return_if_fail (!in_transaction);
 
+	if (!tracker_db_manager_has_enough_space ()) {
+		g_set_error (error, TRACKER_SPARQL_ERROR, TRACKER_SPARQL_ERROR_NO_SPACE,
+			"There is not enough space on the file system for update operations");
+		return;
+	}
+
 	resource_time = time (NULL);
 
 	if (update_buffer.resource_cache == NULL) {
@@ -2064,34 +2199,45 @@ tracker_data_begin_db_transaction (void)
 
 	tracker_db_interface_start_transaction (iface);
 
+	if (!in_journal_replay) {
+		tracker_db_journal_start_transaction (resource_time);
+	}
+
+	iface = tracker_db_manager_get_db_interface ();
+
 	in_transaction = TRUE;
 }
 
 void
-tracker_data_begin_db_transaction_for_replay (time_t time)
+tracker_data_begin_transaction_for_replay (time_t time, GError **error)
 {
 	in_journal_replay = TRUE;
-	tracker_data_begin_db_transaction ();
+	tracker_data_begin_transaction (error);
 	resource_time = time;
 }
 
 void
-tracker_data_commit_db_transaction (void)
+tracker_data_commit_transaction (GError **error)
 {
 	TrackerDBInterface *iface;
+	GError *actual_error = NULL;
 
 	g_return_if_fail (in_transaction);
 
+	iface = tracker_db_manager_get_db_interface ();
+
+	tracker_data_update_buffer_flush (&actual_error);
+	if (actual_error) {
+		g_propagate_error (error, actual_error);
+		return;
+	}
+
 	in_transaction = FALSE;
 
-	tracker_data_update_buffer_flush (NULL);
-
-#if HAVE_TRACKER_FTS
-	if (update_buffer.fts_ever_updated) {
-		tracker_fts_update_commit ();
-		update_buffer.fts_ever_updated = FALSE;
+	if (!in_journal_replay) {
+		tracker_db_journal_commit_db_transaction ();
 	}
-#endif
+	resource_time = 0;
 
 	if (update_buffer.class_counts) {
 		/* successful transaction, no need to rollback class counts,
@@ -2099,7 +2245,12 @@ tracker_data_commit_db_transaction (void)
 		g_hash_table_remove_all (update_buffer.class_counts);
 	}
 
-	iface = tracker_db_manager_get_db_interface ();
+#if HAVE_TRACKER_FTS
+	if (update_buffer.fts_ever_updated) {
+		tracker_fts_update_commit ();
+		update_buffer.fts_ever_updated = FALSE;
+	}
+#endif
 
 	tracker_db_interface_end_db_transaction (iface);
 
@@ -2111,7 +2262,7 @@ tracker_data_commit_db_transaction (void)
 }
 
 void
-tracker_data_notify_db_transaction (void)
+tracker_data_notify_transaction (void)
 {
 	if (commit_callbacks) {
 		guint n;
@@ -2290,7 +2441,7 @@ tracker_data_delete_resource_description (const gchar *graph,
 					continue;
 				}
 
-				if (strcmp (tracker_property_get_uri (property), RDF_PREFIX "type") == 0) {
+				if (property == tracker_ontologies_get_rdf_type ()) {
 					/* Do not delete rdf:type statements */
 					continue;
 				}
@@ -2380,58 +2531,19 @@ tracker_data_delete_resource_description (const gchar *graph,
 	g_free (urn);
 }
 
-void
-tracker_data_begin_transaction (GError **error)
-{
-	TrackerDBInterface *iface;
-
-	if (!tracker_db_manager_has_enough_space ()) {
-		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_NO_SPACE,
-			"There is not enough space on the file system for update operations");
-		return;
-	}
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	resource_time = time (NULL);
-	tracker_db_interface_execute_query (iface, NULL, "SAVEPOINT sparql");
-	tracker_db_journal_start_transaction (resource_time);
-}
-
-void
-tracker_data_commit_transaction (GError **error)
-{
-	TrackerDBInterface *iface;
-	GError *actual_error = NULL;
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	tracker_data_update_buffer_flush (&actual_error);
-	if (actual_error) {
-		g_propagate_error (error, actual_error);
-		return;
-	}
-
-	tracker_db_journal_commit_db_transaction ();
-	resource_time = 0;
-	tracker_db_interface_execute_query (iface, NULL, "RELEASE sparql");
-
-	if (update_buffer.class_counts) {
-		/* successful transaction, no need to rollback class counts,
-		   so remove them */
-		g_hash_table_remove_all (update_buffer.class_counts);
-	}
-}
-
 void
 tracker_data_rollback_transaction (void)
 {
 	TrackerDBInterface *iface;
 
+	g_return_if_fail (in_transaction);
+
+	in_transaction = FALSE;
+
 	iface = tracker_db_manager_get_db_interface ();
 
 	tracker_data_update_buffer_clear ();
-	tracker_db_interface_execute_query (iface, NULL, "ROLLBACK TO sparql");
+	tracker_db_interface_execute_query (iface, NULL, "ROLLBACK");
 	tracker_db_journal_rollback_transaction ();
 
 	if (rollback_callbacks) {
@@ -2495,6 +2607,19 @@ tracker_data_update_sparql_blank (const gchar  *update,
 	return update_sparql (update, TRUE, error);
 }
 
+void
+tracker_data_load_turtle_file (GFile   *file,
+                               GError **error)
+{
+	gchar *path;
+
+	g_return_if_fail (G_IS_FILE (file) && g_file_is_native (file));
+
+	path = g_file_get_path (file);
+	tracker_turtle_reader_load (path, error);
+	g_free (path);
+}
+
 void
 tracker_data_sync (void)
 {
@@ -2540,10 +2665,12 @@ ontology_transaction_end (GList *ontology_queue,
 	GList *l;
 	const gchar *ontology_uri = NULL;
 
+	tracker_data_ontology_process_changes_pre_db (seen_classes, seen_properties);
+
 	/* Perform ALTER-TABLE and CREATE-TABLE calls for all that are is_new */
 	tracker_data_ontology_import_into_db (TRUE);
 
-	tracker_data_ontology_process_changes (seen_classes, seen_properties);
+	tracker_data_ontology_process_changes_post_db (seen_classes, seen_properties);
 
 	for (l = ontology_queue; l; l = l->next) {
 		QueuedStatement *queued = ontology_queue->data;
@@ -2564,6 +2691,8 @@ ontology_transaction_end (GList *ontology_queue,
 
 	}
 
+	tracker_data_ontology_process_changes_post_import (seen_classes, seen_properties);
+
 	/* Update the nao:lastModified in the database */
 	if (ontology_uri) {
 		TrackerOntology *ontology;
@@ -2653,9 +2782,7 @@ tracker_data_replay_journal (GHashTable          *classes,
 	GPtrArray *seen_classes = NULL;
 	GPtrArray *seen_properties = NULL;
 
-	tracker_data_begin_db_transaction_for_replay (0);
-
-	rdf_type = tracker_ontologies_get_property_by_uri (RDF_PREFIX "type");
+	rdf_type = tracker_ontologies_get_rdf_type ();
 
 	tracker_db_journal_reader_init (NULL);
 
@@ -2677,7 +2804,7 @@ tracker_data_replay_journal (GHashTable          *classes,
 			tracker_db_journal_reader_get_resource (&id, &uri);
 
 			if (in_ontology) {
-				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), (gpointer) uri);
+				g_hash_table_insert (id_uri_map, GINT_TO_POINTER (id), g_strdup (uri));
 				continue;
 			}
 
@@ -2709,10 +2836,14 @@ tracker_data_replay_journal (GHashTable          *classes,
 			}
 
 		} else if (type == TRACKER_DB_JOURNAL_START_ONTOLOGY_TRANSACTION) {
+			tracker_data_begin_transaction_for_replay (tracker_db_journal_reader_get_time (), NULL);
 			in_ontology = TRUE;
 		} else if (type == TRACKER_DB_JOURNAL_START_TRANSACTION) {
-			resource_time = tracker_db_journal_reader_get_time ();
+			tracker_data_begin_transaction_for_replay (tracker_db_journal_reader_get_time (), NULL);
 		} else if (type == TRACKER_DB_JOURNAL_END_TRANSACTION) {
+			GError *new_error = NULL;
+			tracker_data_update_buffer_might_flush (&new_error);
+
 			if (in_ontology) {
 				ontology_transaction_end (ontology_queue, seen_classes, seen_properties);
 				g_list_foreach (ontology_queue, (GFunc) free_queued_statement, NULL);
@@ -2723,13 +2854,12 @@ tracker_data_replay_journal (GHashTable          *classes,
 				seen_classes = NULL;
 				tracker_data_ontology_free_seen (seen_properties);
 				seen_properties = NULL;
-			} else {
-				GError *new_error = NULL;
-				tracker_data_update_buffer_might_flush (&new_error);
-				if (new_error) {
-					g_warning ("Journal replay error: '%s'", new_error->message);
-					g_clear_error (&new_error);
-				}
+			}
+
+			tracker_data_commit_transaction (&new_error);
+			if (new_error) {
+				g_warning ("Journal replay error: '%s'", new_error->message);
+				g_clear_error (&new_error);
 			}
 		} else if (type == TRACKER_DB_JOURNAL_INSERT_STATEMENT) {
 			GError *new_error = NULL;
@@ -2970,6 +3100,4 @@ tracker_data_replay_journal (GHashTable          *classes,
 	} else {
 		tracker_db_journal_reader_shutdown ();
 	}
-
-	tracker_data_commit_db_transaction ();
 }
