@@ -28,10 +28,14 @@
 
 #include <libtracker-common/tracker-common.h>
 
+#include <libtracker-sparql/tracker-sparql.h>
+
 #if HAVE_TRACKER_FTS
 #include <libtracker-fts/tracker-fts.h>
 #endif
 
+#include "tracker-collation.h"
+
 #include "tracker-db-interface-sqlite.h"
 
 struct TrackerDBInterface {
@@ -44,6 +48,8 @@ struct TrackerDBInterface {
 
 	GSList *function_data;
 
+	gpointer collator;
+
 	guint ro : 1;
 #if HAVE_TRACKER_FTS
 	guint fts_initialized : 1;
@@ -56,14 +62,14 @@ struct TrackerDBInterfaceClass {
 };
 
 struct TrackerDBCursor {
-	GObject parent_instance;
+	TrackerSparqlCursor parent_instance;
 	sqlite3_stmt *stmt;
 	TrackerDBStatement *ref_stmt;
 	gboolean finished;
 };
 
 struct TrackerDBCursorClass {
-	GObjectClass parent_class;
+	TrackerSparqlCursorClass parent_class;
 };
 
 struct TrackerDBStatement {
@@ -93,12 +99,11 @@ G_DEFINE_TYPE (TrackerDBInterface, tracker_db_interface, G_TYPE_OBJECT)
 
 G_DEFINE_TYPE (TrackerDBStatement, tracker_db_statement, G_TYPE_OBJECT)
 
-G_DEFINE_TYPE (TrackerDBCursor, tracker_db_cursor, G_TYPE_OBJECT)
+G_DEFINE_TYPE (TrackerDBCursor, tracker_db_cursor, TRACKER_SPARQL_TYPE_CURSOR)
 
 void
 tracker_db_interface_sqlite_enable_shared_cache (void)
 {
-	sqlite3_config (SQLITE_CONFIG_MULTITHREAD);
 	sqlite3_enable_shared_cache (1);
 }
 
@@ -482,19 +487,33 @@ check_interrupt (void *user_data)
 static void
 open_database (TrackerDBInterface *db_interface)
 {
+	int mode;
+
 	g_assert (db_interface->filename != NULL);
 
 	if (!db_interface->ro) {
-		if (sqlite3_open (db_interface->filename, &db_interface->db) != SQLITE_OK) {
-			g_critical ("Could not open sqlite3 database:'%s'", db_interface->filename);
-		} else {
-			g_message ("Opened sqlite3 database:'%s'", db_interface->filename);
-		}
+		mode = SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE;
 	} else {
-		if (sqlite3_open_v2 (db_interface->filename, &db_interface->db, SQLITE_OPEN_READONLY, NULL) != SQLITE_OK) {
-			g_critical ("Could not open sqlite3 database:'%s'", db_interface->filename);
-		} else {
-			g_message ("Opened sqlite3 database:'%s'", db_interface->filename);
+		mode = SQLITE_OPEN_READONLY;
+	}
+
+	if (sqlite3_open_v2 (db_interface->filename, &db_interface->db, mode | SQLITE_OPEN_NOMUTEX, NULL) != SQLITE_OK) {
+		g_critical ("Could not open sqlite3 database:'%s'", db_interface->filename);
+	} else {
+		g_message ("Opened sqlite3 database:'%s'", db_interface->filename);
+	}
+
+	/* Set our unicode collation function */
+	if (!db_interface->collator) {
+		db_interface->collator = tracker_collation_init ();
+		if (sqlite3_create_collation (db_interface->db,
+		                              TRACKER_COLLATION_NAME,
+		                              SQLITE_UTF8,
+		                              db_interface->collator,
+		                              tracker_collation_utf8) != SQLITE_OK)
+		{
+			g_critical ("Couldn't set collation function: %s",
+			            sqlite3_errmsg (db_interface->db));
 		}
 	}
 
@@ -642,6 +661,8 @@ tracker_db_interface_sqlite_finalize (GObject *object)
 
 	g_free (db_interface->filename);
 
+	tracker_collation_shutdown (db_interface->collator);
+
 	G_OBJECT_CLASS (tracker_db_interface_parent_class)->finalize (object);
 }
 
@@ -766,6 +787,11 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 
 	stmt = g_hash_table_lookup (db_interface->dynamic_statements, full_query);
 
+	if (stmt && stmt->stmt_is_sunk) {
+		/* prepared statement is still in use, create new one */
+		stmt = NULL;
+	}
+
 	if (!stmt) {
 		sqlite3_stmt *sqlite_stmt;
 		int retval;
@@ -796,9 +822,12 @@ tracker_db_interface_create_statement (TrackerDBInterface  *db_interface,
 
 		stmt = tracker_db_statement_sqlite_new (db_interface, sqlite_stmt);
 
-		g_hash_table_insert (db_interface->dynamic_statements,
-		                     (gpointer) sqlite3_sql (sqlite_stmt),
-		                     stmt);
+		/* use replace instead of insert to make sure we store the string that
+		   belongs to the right sqlite statement to ensure the lifetime of the string
+		   matches the statement */
+		g_hash_table_replace (db_interface->dynamic_statements,
+		                      (gpointer) sqlite3_sql (sqlite_stmt),
+		                      stmt);
 	} else {
 		tracker_db_statement_sqlite_reset (stmt);
 	}
@@ -965,34 +994,6 @@ tracker_db_interface_sqlite_new_ro (const gchar *filename)
 	                     NULL);
 }
 
-static gint
-collation_function (gpointer      data,
-                    int           len1,
-                    gconstpointer str1,
-                    int           len2,
-                    gconstpointer str2)
-{
-	TrackerDBCollationFunc func;
-
-	func = (TrackerDBCollationFunc) data;
-
-	return (func) ((gchar *) str1, len1, (gchar *) str2, len2);
-}
-
-gboolean
-tracker_db_interface_sqlite_set_collation_function (TrackerDBInterface       *interface,
-                                                    const gchar              *name,
-                                                    TrackerDBCollationFunc    func)
-{
-	gint result;
-
-	g_return_val_if_fail (TRACKER_IS_DB_INTERFACE (interface), FALSE);
-
-	result = sqlite3_create_collation (interface->db, name, SQLITE_UTF8, func, &collation_function);
-
-	return (result == SQLITE_OK);
-}
-
 gint64
 tracker_db_interface_sqlite_get_last_insert_id (TrackerDBInterface *interface)
 {
@@ -1061,12 +1062,61 @@ tracker_db_cursor_finalize (GObject *object)
 	G_OBJECT_CLASS (tracker_db_cursor_parent_class)->finalize (object);
 }
 
+static void
+tracker_db_cursor_iter_next_thread (GSimpleAsyncResult *res,
+                                    GObject            *object,
+                                    GCancellable       *cancellable)
+{
+	/* run in thread */
+
+	GError *error = NULL;
+	gboolean result;
+
+	result = tracker_db_cursor_iter_next (TRACKER_DB_CURSOR (object), cancellable, &error);
+	if (error) {
+		g_simple_async_result_set_from_error (res, error);
+	} else {
+		g_simple_async_result_set_op_res_gboolean (res, result);
+	}
+}
+
+static void
+tracker_db_cursor_iter_next_async (TrackerDBCursor     *cursor,
+                                   GCancellable        *cancellable,
+                                   GAsyncReadyCallback  callback,
+                                   gpointer             user_data)
+{
+	GSimpleAsyncResult *res;
+
+	res = g_simple_async_result_new (G_OBJECT (cursor), callback, user_data, tracker_db_cursor_iter_next_async);
+	g_simple_async_result_run_in_thread (res, tracker_db_cursor_iter_next_thread, 0, cancellable);
+}
+
+static gboolean
+tracker_db_cursor_iter_next_finish (TrackerDBCursor  *cursor,
+                                    GAsyncResult     *res,
+                                    GError          **error)
+{
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error)) {
+		return FALSE;
+	}
+	return g_simple_async_result_get_op_res_gboolean (G_SIMPLE_ASYNC_RESULT (res));
+}
+
 static void
 tracker_db_cursor_class_init (TrackerDBCursorClass *class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (class);
+	TrackerSparqlCursorClass *sparql_cursor_class = TRACKER_SPARQL_CURSOR_CLASS (class);
 
 	object_class->finalize = tracker_db_cursor_finalize;
+
+	sparql_cursor_class->get_n_columns = (gint (*) (TrackerSparqlCursor *)) tracker_db_cursor_get_n_columns;
+	sparql_cursor_class->get_string = (const gchar * (*) (TrackerSparqlCursor *, gint, glong*)) tracker_db_cursor_get_string;
+	sparql_cursor_class->next = (gboolean (*) (TrackerSparqlCursor *, GCancellable *, GError **)) tracker_db_cursor_iter_next;
+	sparql_cursor_class->next_async = (void (*) (TrackerSparqlCursor *, GCancellable *, GAsyncReadyCallback, gpointer)) tracker_db_cursor_iter_next_async;
+	sparql_cursor_class->next_finish = (gboolean (*) (TrackerSparqlCursor *, GAsyncResult *, GError **)) tracker_db_cursor_iter_next_finish;
+	sparql_cursor_class->rewind = (void (*) (TrackerSparqlCursor *)) tracker_db_cursor_rewind;
 }
 
 static TrackerDBCursor *
@@ -1143,6 +1193,7 @@ tracker_db_cursor_rewind (TrackerDBCursor *cursor)
 	g_return_if_fail (TRACKER_IS_DB_CURSOR (cursor));
 
 	sqlite3_reset (cursor->stmt);
+	cursor->finished = FALSE;
 }
 
 gboolean
@@ -1234,7 +1285,7 @@ tracker_db_cursor_get_double (TrackerDBCursor *cursor,  guint column)
 
 
 const gchar*
-tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, gint *length)
+tracker_db_cursor_get_string (TrackerDBCursor *cursor,  guint column, glong *length)
 {
 	if (length) {
 		sqlite3_value *val = sqlite3_column_value (cursor->stmt, column);
