@@ -37,11 +37,13 @@
 #include "tracker-dbus.h"
 #include "tracker-marshal.h"
 #include "tracker-resources.h"
-#include "tracker-resource-class.h"
 #include "tracker-events.h"
 #include "tracker-writeback.h"
 #include "tracker-store.h"
 
+#define TRACKER_GRAPH_UPDATED_IMMEDIATE_EMIT_AT   1000
+#define TRACKER_SIGNALS_SECONDS_PER_EMIT          1
+
 #define RDF_PREFIX TRACKER_RDF_PREFIX
 #define RDF_TYPE RDF_PREFIX "type"
 
@@ -76,11 +78,13 @@ G_DEFINE_TYPE(TrackerResources, tracker_resources, G_TYPE_OBJECT)
 
 enum {
 	WRITEBACK,
+	GRAPHUPDATED,
 	LAST_SIGNAL
 };
 
 typedef struct {
-	GSList *event_sources;
+	DBusConnection *connection;
+	guint signal_timeout;
 } TrackerResourcesPrivate;
 
 typedef struct {
@@ -96,21 +100,8 @@ typedef struct {
 } InThreadPtr;
 
 static void tracker_resources_finalize (GObject *object);
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
-static void
-free_event_sources (TrackerResourcesPrivate *priv)
-{
-	if (priv->event_sources) {
-		g_slist_foreach (priv->event_sources,
-		                 (GFunc) g_object_unref, NULL);
-		g_slist_free (priv->event_sources);
-
-		priv->event_sources = NULL;
-	}
-}
-
 static void
 tracker_resources_class_init (TrackerResourcesClass *klass)
 {
@@ -128,7 +119,20 @@ tracker_resources_class_init (TrackerResourcesClass *klass)
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__BOXED,
 		              G_TYPE_NONE, 1,
-		              TRACKER_TYPE_STR_STRV_MAP);
+		              TRACKER_TYPE_INT_ARRAY_MAP);
+
+	/* This is just for introspection to work */
+	signals[GRAPHUPDATED] =
+		g_signal_new ("graph-updated",
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerResourcesClass, graph_updated),
+		              NULL, NULL,
+		              tracker_marshal_VOID__STRING_BOXED_BOXED,
+		              G_TYPE_NONE, 3,
+		              G_TYPE_STRING,
+		              TRACKER_TYPE_FOUR_INT_ARRAY,
+		              TRACKER_TYPE_FOUR_INT_ARRAY);
 
 	g_type_class_add_private (object_class, sizeof (TrackerResourcesPrivate));
 }
@@ -140,9 +144,16 @@ tracker_resources_init (TrackerResources *object)
 }
 
 TrackerResources *
-tracker_resources_new (void)
+tracker_resources_new (DBusGConnection *connection)
 {
-	return g_object_new (TRACKER_TYPE_RESOURCES, NULL);
+	TrackerResourcesPrivate *priv;
+	TrackerResources *resources = g_object_new (TRACKER_TYPE_RESOURCES, NULL);
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (resources);
+
+	priv->connection = dbus_connection_ref (dbus_g_connection_get_connection (connection));
+
+	return resources;
 }
 
 /*
@@ -183,6 +194,7 @@ tracker_resources_load (TrackerResources         *object,
 	TrackerDBusMethodInfo   *info;
 	guint                    request_id;
 	GFile  *file;
+	gchar *sender;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
@@ -201,9 +213,12 @@ tracker_resources_load (TrackerResources         *object,
 	info->request_id = request_id;
 	info->context = context;
 
-	tracker_store_queue_turtle_import (file, turtle_import_callback,
-	                                   info, destroy_method_info);
+	sender = dbus_g_method_get_sender (context);
 
+	tracker_store_queue_turtle_import (file, turtle_import_callback,
+	                                   sender, info, destroy_method_info);
+
+	g_free (sender);
 	g_object_unref (file);
 }
 
@@ -396,7 +411,7 @@ tracker_resources_sparql_update (TrackerResources        *self,
 
 	sender = dbus_g_method_get_sender (context);
 
-	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_HIGH, FALSE,
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_HIGH,
 	                             update_callback, sender,
 	                             info, destroy_method_info);
 
@@ -503,184 +518,248 @@ tracker_resources_batch_sparql_update (TrackerResources          *self,
 
 	sender = dbus_g_method_get_sender (context);
 
-	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_LOW, TRUE,
+	tracker_store_sparql_update (update, TRACKER_STORE_PRIORITY_LOW,
 	                             update_callback, sender,
 	                             info, destroy_method_info);
 
 	g_free (sender);
 }
 
-static void
-batch_commit_callback (gpointer user_data)
-{
-	TrackerDBusMethodInfo *info = user_data;
-
-	tracker_data_sync ();
-
-	tracker_dbus_request_success (info->request_id,
-	                              info->context);
-
-	dbus_g_method_return (info->context);
-}
-
 void
 tracker_resources_batch_commit (TrackerResources         *self,
                                 DBusGMethodInvocation    *context,
                                 GError                  **error)
 {
-	TrackerDBusMethodInfo *info;
-	guint                 request_id;
-	gchar                 *sender;
-
-	request_id = tracker_dbus_get_next_request_id ();
-
-	tracker_dbus_request_new (request_id,
-	                          context,
-	                          "%s()",
-	                          __FUNCTION__);
-
-	info = g_slice_new (TrackerDBusMethodInfo);
-
-	info->request_id = request_id;
-	info->context = context;
-
-	sender = dbus_g_method_get_sender (context);
-
-	tracker_store_queue_commit (batch_commit_callback, sender, info,
-	                            destroy_method_info);
-
-	g_free (sender);
+	/* no longer needed */
 }
 
+static void
+foreach_add_to_iter (gint graph_id,
+                     gint subject_id,
+                     gint pred_id,
+                     gint object_id,
+                     gpointer user_data)
+{
+	DBusMessageIter *array_iter = user_data;
+	DBusMessageIter struct_iter;
+
+	dbus_message_iter_open_container (array_iter, DBUS_TYPE_STRUCT, NULL, &struct_iter);
+	dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_INT32, &graph_id);
+	dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_INT32, &subject_id);
+	dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_INT32, &pred_id);
+	dbus_message_iter_append_basic (&struct_iter, DBUS_TYPE_INT32, &object_id);
+	dbus_message_iter_close_container (array_iter, &struct_iter);
+}
+
+static gboolean
+emit_graph_updated (TrackerResources *self,
+                    TrackerClass     *class)
+{
+	if (tracker_class_has_insert_events (class) || tracker_class_has_delete_events (class)) {
+		TrackerResourcesPrivate *priv;
+		DBusMessageIter iter, deletes_iter, inserts_iter;
+		DBusMessage *message;
+		const gchar *class_uri;
+
+		priv = TRACKER_RESOURCES_GET_PRIVATE (self);
+
+		message = dbus_message_new_signal (TRACKER_RESOURCES_PATH,
+		                                   TRACKER_RESOURCES_INTERFACE,
+		                                   "GraphUpdated");
+
+		class_uri = tracker_class_get_uri (class);
+
+		dbus_message_iter_init_append (message, &iter);
+		dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &class_uri);
+
+		dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY,
+		                                  "(iiii)", &deletes_iter);
+
+		tracker_class_foreach_delete_event (class, foreach_add_to_iter, &deletes_iter);
+
+		dbus_message_iter_close_container (&iter, &deletes_iter);
+
+
+		dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY,
+		                                  "(iiii)", &inserts_iter);
+
+		tracker_class_foreach_insert_event (class, foreach_add_to_iter, &inserts_iter);
+
+		dbus_message_iter_close_container (&iter, &inserts_iter);
+
+		dbus_connection_send (priv->connection, message, NULL);
+
+		dbus_message_unref (message);
+
+		tracker_class_reset_ready_events (class);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+on_emit_signals (gpointer user_data)
+{
+	TrackerResources *resources = user_data;
+	TrackerResourcesPrivate *priv;
+	GHashTableIter iter;
+	gpointer key, value;
+	gboolean had_any = FALSE;
+	GHashTable *writebacks;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (resources);
+
+	/* Class signal feature */
+	tracker_events_classes_iter (&iter);
+
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		TrackerClass *class = key;
+		if (emit_graph_updated (user_data, class)) {
+			had_any = TRUE;
+		}
+	}
+
+	/* Reset counter */
+	tracker_events_get_total (TRUE);
+
+
+	/* Writeback feature */
+	writebacks = tracker_writeback_get_ready ();
+
+	if (writebacks) {
+		had_any = TRUE;
+		g_signal_emit (resources, signals[WRITEBACK], 0, writebacks);
+	}
+
+	tracker_writeback_reset_ready ();
+
+	if (!had_any)
+		priv->signal_timeout = 0;
+
+	return had_any;
+}
 
 static void
 on_statements_committed (gpointer user_data)
 {
 	TrackerResources *resources = user_data;
-	GArray *events;
-	GHashTable *writebacks;
 	TrackerResourcesPrivate *priv;
+	GHashTableIter iter;
+	gpointer key, value;
+
+	/* Class signal feature */
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (resources);
 
-	/* Class signals feature */
-	events = tracker_events_get_pending ();
+	/* Class signal feature */
+	tracker_events_classes_iter (&iter);
 
-	/* Do not call tracker_events_reset before calling tracker_resource_class_emit_events
-	   as we're reusing the same strings without copies */
-
-	if (events) {
-		GSList *event_sources, *l;
-		guint i;
-		GHashTable *to_emit = NULL;
-
-		event_sources = priv->event_sources;
-
-		for (i = 0; i < events->len; i++) {
-			TrackerEvent *event;
-
-			event = &g_array_index (events, TrackerEvent, i);
-
-			for (l = event_sources; l; l = l->next) {
-				TrackerResourceClass *class_ = l->data;
-				if (g_strcmp0 (tracker_class_get_uri (event->class), tracker_resource_class_get_rdf_class (class_)) == 0) {
-					tracker_resource_class_add_event (class_, event->subject, event->predicate, event->type);
-					if (!to_emit) {
-						to_emit = g_hash_table_new (NULL, NULL);
-					}
-					g_hash_table_insert (to_emit, class_, class_);
-				}
-			}
-		}
-
-		if (to_emit) {
-			GHashTableIter iter;
-			gpointer key, value;
-
-			g_hash_table_iter_init (&iter, to_emit);
-
-			while (g_hash_table_iter_next (&iter, &key, &value)) {
-				TrackerResourceClass *class_ = key;
-				tracker_resource_class_emit_events (class_);
-			}
-
-			g_hash_table_destroy (to_emit);
-		}
+	while (g_hash_table_iter_next (&iter, &key, &value)) {
+		TrackerClass *class = key;
+		tracker_class_transact_events (class);
 	}
 
-	tracker_events_reset ();
+	if (priv->signal_timeout == 0) {
+		priv->signal_timeout = g_timeout_add_seconds (TRACKER_SIGNALS_SECONDS_PER_EMIT,
+		                                              on_emit_signals,
+		                                              user_data);
+	}
 
 	/* Writeback feature */
-	writebacks = tracker_writeback_get_pending ();
-
-	if (writebacks) {
-		g_signal_emit (resources, signals[WRITEBACK], 0, writebacks);
-
-	}
-
-	tracker_writeback_reset ();
+	tracker_writeback_transact ();
 }
 
 
 static void
 on_statements_rolled_back (gpointer user_data)
 {
-	tracker_events_reset ();
-	tracker_writeback_reset ();
+	tracker_events_reset_pending ();
+	tracker_writeback_reset_pending ();
 }
 
 static void
-on_statement_inserted (const gchar *graph,
-                       const gchar *subject,
-                       const gchar *predicate,
-                       const gchar *object,
-                       GPtrArray   *rdf_types,
-                       gpointer user_data)
-{
-	if (g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
-		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_ADD);
-	} else {
-		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
-	}
-
-	/* For predicates it's always update here */
-	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
-}
-
-static void
-on_statement_deleted (const gchar *graph,
-                      const gchar *subject,
-                      const gchar *predicate,
-                      const gchar *object,
-                      GPtrArray   *rdf_types,
-                      gpointer user_data)
-{
-	if (g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
-		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_DELETE);
-	} else {
-		tracker_events_insert (subject, predicate, object, rdf_types, TRACKER_DBUS_EVENTS_TYPE_UPDATE);
-	}
-
-	/* For predicates it's always delete here */
-	tracker_writeback_check (graph, subject, predicate, object, rdf_types);
-}
-
-void
-tracker_resources_prepare (TrackerResources *object,
-                           GSList           *event_sources)
+check_graph_updated_signal (TrackerResources *object)
 {
 	TrackerResourcesPrivate *priv;
 
 	priv = TRACKER_RESOURCES_GET_PRIVATE (object);
 
-	free_event_sources (priv);
+	/* Check for whether we need an immediate emit */
+	if (tracker_events_get_total (FALSE) > TRACKER_GRAPH_UPDATED_IMMEDIATE_EMIT_AT) {
+		gpointer key, value;
+		GHashTableIter iter;
+
+		tracker_events_classes_iter (&iter);
+		while (g_hash_table_iter_next (&iter, &key, &value)) {
+			TrackerClass *class = key;
+			emit_graph_updated (object, class);
+		}
+
+		/* Reset counter */
+		tracker_events_get_total (TRUE);
+	}
+}
+
+static void
+on_statement_inserted (gint         graph_id,
+                       const gchar *graph,
+                       gint         subject_id,
+                       const gchar *subject,
+                       gint         pred_id,
+                       gint         object_id,
+                       const gchar *object,
+                       GPtrArray   *rdf_types,
+                       gpointer     user_data)
+{
+	TrackerResourcesPrivate *priv;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (user_data);
+
+	tracker_events_add_insert (graph_id, subject_id, subject, pred_id,
+	                           object_id, object, rdf_types);
+	tracker_writeback_check (graph_id, graph, subject_id,
+	                         subject, pred_id, object_id,
+	                         object, rdf_types);
+	check_graph_updated_signal (user_data);
+}
+
+static void
+on_statement_deleted (gint         graph_id,
+                      const gchar *graph,
+                      gint         subject_id,
+                      const gchar *subject,
+                      gint         pred_id,
+                      gint         object_id,
+                      const gchar *object,
+                      GPtrArray   *rdf_types,
+                      gpointer     user_data)
+{
+	TrackerResourcesPrivate *priv;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (user_data);
+
+	tracker_events_add_delete (graph_id, subject_id, subject, pred_id,
+	                           object_id, object, rdf_types);
+	tracker_writeback_check (graph_id, graph, subject_id,
+	                         subject, pred_id, object_id,
+	                         object, rdf_types);
+	check_graph_updated_signal (user_data);
+}
+
+void
+tracker_resources_prepare (TrackerResources *object)
+{
+	TrackerResourcesPrivate *priv;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (object);
 
 	tracker_data_add_insert_statement_callback (on_statement_inserted, object);
 	tracker_data_add_delete_statement_callback (on_statement_deleted, object);
 	tracker_data_add_commit_statement_callback (on_statements_committed, object);
 	tracker_data_add_rollback_statement_callback (on_statements_rolled_back, object);
 
-	priv->event_sources = event_sources;
 }
 
 static void
@@ -695,7 +774,11 @@ tracker_resources_finalize (GObject      *object)
 	tracker_data_remove_commit_statement_callback (on_statements_committed, object);
 	tracker_data_remove_rollback_statement_callback (on_statements_rolled_back, object);
 
-	free_event_sources (priv);
+	if (priv->signal_timeout != 0) {
+		g_source_remove (priv->signal_timeout);
+	}
+
+	dbus_connection_unref (priv->connection);
 
 	G_OBJECT_CLASS (tracker_resources_parent_class)->finalize (object);
 }
