@@ -57,6 +57,8 @@ const TestInfo tests[] = {
 	{ "algebra/filter-scope-1", "algebra/data-2", FALSE },
 	{ "algebra/filter-in-1", "algebra/data-2", FALSE },
 	{ "algebra/filter-in-2", "algebra/data-2", FALSE },
+	{ "algebra/filter-in-3", "algebra/data-2", FALSE },
+	{ "algebra/filter-in-4", "algebra/data-2", FALSE },
 	{ "algebra/var-scope-join-1", "algebra/var-scope-join-1", FALSE },
 	{ "anon/query", "anon/data", FALSE },
 	{ "ask/ask-1", "ask/data", FALSE },
@@ -254,9 +256,7 @@ test_sparql_query (gconstpointer test_data)
 	/* load data set */
 	data_filename = g_strconcat (data_prefix, ".ttl", NULL);
 	if (g_file_test (data_filename, G_FILE_TEST_IS_REGULAR)) {
-		tracker_data_begin_db_transaction ();
 		tracker_turtle_reader_load (data_filename, &error);
-		tracker_data_commit_db_transaction ();
 		g_assert_no_error (error);
 	} else {
 		/* no .ttl available, assume .rq with SPARQL Update */
@@ -268,9 +268,7 @@ test_sparql_query (gconstpointer test_data)
 		g_file_get_contents (data_filename, &data, NULL, &error);
 		g_assert_no_error (error);
 
-		tracker_data_begin_db_transaction ();
 		tracker_data_update_sparql (data, &error);
-		tracker_data_commit_db_transaction ();
 		if (test_info->expect_update_error) {
 			g_assert (error != NULL);
 			g_clear_error (&error);
